{% extends "djimporter/base.html" %}
{% load i18n %}

{% block content %}
<table class="table table-striped">
    <thead>
        <tr>
          <th scope="col">{% trans 'ID' %}</th>
            <th scope="col">{% trans 'Created' %}</th>
            <th scope="col">{% trans 'File' %}</th>
            <th scope="col">{% trans 'User' %}</th>
            <th scope="col">{% trans 'Status' %}</th>
            <th scope="col"></th>
        </tr>
    </thead>
    <tbody>
        {% for log in object_list %}
        <tr>
          <td>{{ log.id }}</td>
          <td>{{ log.created_at }}</td>
          <td>{{ log.input_file }}</td>
          <td>{{ log.user }}</td>
          <td>{{ log.status }}</td>
          <td>
            <a class="btn btn-link" href="{% url url_detail log.id %}"><i class="fas fa-list-ol"></i></a>
            <a class="btn text-danger" href="{% url url_delete log.id %}"><i class="fas fa-trash"></i></a>
          </td>
        </tr>
        {% empty %}
        <tr colspan="">No logs yet.</tr>
        {% endfor %}
    </tbody>
</table>
{% endblock %}
