# Generated by Django 3.2.17 on 2023-05-15 13:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djimporter', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='importlog',
            name='user',
            field=models.CharField(max_length=512),
        ),
    ]
