{% extends "djimporter/base.html" %}
{% load i18n %}

{% block content %}

  <form method="post">{% csrf_token %}
      <p>{% blocktrans with pk=object.pk %}Are you sure that you want remove the log: "{{ pk }}"?{% endblocktrans %}</p>
      <input type="submit" class="btn btn-danger" value="{% trans 'Confirm' %}">
  </form>

{% endblock %}
