{% load i18n %}
<p class="text-info"><b>{% trans 'Columns to import' %}:</b> 
    {% for field in importer.get_user_visible_fields %}
    <span class="badge badge-secondary">{{field}}</span>
    {% endfor %}
</p>

<p class="text-info"><b>{% trans 'Csv delimiter' %}</b> <span class="badge badge-secondary">{{ importer.get_delimiter }}</span></p>
<p class="text-info"><b>{% trans 'File encoding' %}</b> <span class="badge badge-secondary">UTF-8</span></p>